@@ -26,6 +26,8 @@ High-level Pipelines with plugable back-ends
 import gobject
 import gst
 from elements.smartscale import SmartVideoScale
+from ui import plumber
+from threads import CallbackThread
 
 class SmartBin(gst.Pipeline):
     """
@@ -34,7 +36,7 @@ class SmartBin(gst.Pipeline):
     """
 
     def __init__(self, name, displayname="", has_video=False, has_audio=False,
-                 length=0, width=0, height=0):
+                 length=0, width=0, height=0, is_seekable=False):
         """
         @type name: string
         @param name: The name of the SmartBin (for internal use)
@@ -58,12 +60,10 @@ class SmartBin(gst.Pipeline):
         # Until  basetransform issues are fixed, we use an identity instead
         # of a tee
         if self.has_video:
-            #self.vtee = gst.element_factory_make("tee", "vtee")
-            self.vtee = gst.element_factory_make("identity", "vtee")
+            self.vtee = gst.element_factory_make("tee", "vtee")
             self.add(self.vtee)
         if self.has_audio:
-            #self.atee = gst.element_factory_make("tee", "atee")
-            self.atee = gst.element_factory_make("identity", "atee")
+            self.atee = gst.element_factory_make("tee", "atee")
             self.add(self.atee)
         self._addSource()
         self._connectSource()
@@ -74,6 +74,14 @@ class SmartBin(gst.Pipeline):
         self.tmpvsink = None
         self.recording = False
 
+        # set this to True in subclasses if needed
+        self._seekable = is_seekable
+
+    @property
+    def seekable(self):
+        """True if the bin is seekable and has a duration"""
+        return self._seekable
+
     def _addSource(self):
         """ add the source to self """
         raise NotImplementedError
@@ -88,7 +96,7 @@ class SmartBin(gst.Pipeline):
         Returns False if there was a problem.
         """
         self.debug("asinkthread : %r" % asinkthread)
-        res, state, pending = self.get_state(0)
+        state  = self.get_state(0)[1]
         if state == gst.STATE_PLAYING:
             self.warning("is in PAUSED or higher : %s" % state)
             return False
@@ -98,8 +106,7 @@ class SmartBin(gst.Pipeline):
         if self.has_audio:
             self.asinkthread = asinkthread
             self.add(self.asinkthread)
-            # identity vs tee issue
-            self.atee.get_pad("src").link(self.asinkthread.get_pad("sink"))
+            self.atee.get_request_pad("src%d").link(self.asinkthread.get_pad("sink"))
         return True
 
     def setVideoSinkThread(self, vsinkthread):
@@ -108,7 +115,7 @@ class SmartBin(gst.Pipeline):
         Returns False if there was a problem.
         """
         self.debug("vsinkthread : %r" % vsinkthread)
-        res , state , pending = self.get_state(0)
+        state = self.get_state(0)[1]
         if state == gst.STATE_PLAYING:
             self.warning("is in PAUSED or higher : %s" % state)
             return False
@@ -118,11 +125,7 @@ class SmartBin(gst.Pipeline):
         if self.has_video:
             self.vsinkthread = vsinkthread
             self.add(self.vsinkthread)
-            # identity vs tee issue
-            if self.width and self.height:
-                self.vtee.get_pad("src").link(self.vsinkthread.get_pad("sink"))
-            else:
-                self.vtee.get_pad("src").link(self.vsinkthread.get_pad("sink"))
+            self.vtee.get_request_pad("src%d").link(self.vsinkthread.get_pad("sink"))
         return True
 
     def removeAudioSinkThread(self):
@@ -131,7 +134,7 @@ class SmartBin(gst.Pipeline):
         Returns False if there was a problem.
         """
         self.debug("asinkthread : %r" % self.asinkthread)
-        result, state, pending = self.get_state(0)
+        state = self.get_state(0)[1]
         if state in [gst.STATE_PAUSED, gst.STATE_PLAYING]:
             self.warning("is in PAUSED, not removing audiosink")
             return False
@@ -150,7 +153,7 @@ class SmartBin(gst.Pipeline):
         Returns False if there was a problem.
         """
         self.debug("vsinkthread : %r" % self.vsinkthread)
-        result, state, pending = self.get_state(0)
+        state = self.get_state(0)[1]
         if state in [gst.STATE_PAUSED, gst.STATE_PLAYING]:
             self.warning("is in PAUSED or higher : %s" % state)
             return False
@@ -188,12 +191,6 @@ class SmartBin(gst.Pipeline):
         if not self.removeAudioSinkThread():
             return False
 
-        # FIXME : remove this once BaseTransform is fixed
-        # Also temporarily remove the videosinkthread
-        self.tmpvsink = self.vsinkthread
-        if not self.removeVideoSinkThread():
-            return False
-
         self.debug("creating and adding encoding thread")
         self.encthread = self._makeEncThread(uri, settings)
         if not self.encthread:
@@ -202,18 +199,17 @@ class SmartBin(gst.Pipeline):
         self.add(self.encthread)
         self.debug("encoding thread added")
 
-        # set sync=false on the videosink
-        #self.getRealVideoSink().set_property("sync", False)
+        self.getRealVideoSink().set_property("sync", False)
 
         self.debug("linking vtee to ecnthread:vsink")
         try:
-            self.vtee.get_pad("src").link(self.encthread.get_pad("vsink"))
+            self.vtee.get_request_pad("src%d").link(self.encthread.get_pad("vsink"))
         except:
             return False
 
         self.debug("linking atee to encthread:asink")
         try:
-            self.atee.get_pad("src").link(self.encthread.get_pad("asink"))
+            self.atee.get_request_pad("src%d").link(self.encthread.get_pad("asink"))
         except:
             return False
 
@@ -229,7 +225,7 @@ class SmartBin(gst.Pipeline):
     def stopRecording(self):
         """ stop the recording, removing the encoding thread """
         if self.recording == False:
-            self.error("This bin is not in recording mode !")
+            self.warning("This bin is not in recording mode !")
             return False
 
         self.set_state(gst.STATE_PAUSED)
@@ -243,14 +239,17 @@ class SmartBin(gst.Pipeline):
             self.encthread.set_state(gst.STATE_NULL)
             del self.encthread
             self.encthread = None
-            self.setAudioSinkThread(self.tmpasink)
-            self.setVideoSinkThread(self.tmpvsink)
+            if self.tmpasink:
+                self.setAudioSinkThread(self.tmpasink)
+            if self.tmpvsink:
+                self.setVideoSinkThread(self.tmpvsink)
             self.tmpasink = None
             self.tmpvsink = None
 
         self.getRealVideoSink().set_property("sync", True)
 
         self.recording = False
+        self.set_state(gst.STATE_PAUSED)
         return True
 
     def getSettings(self):
@@ -274,9 +273,9 @@ class SmartBin(gst.Pipeline):
         # TODO : verify if encoders take video/x-raw-yuv and audio/x-raw-int
         # TODO : use video/audio settings !
         # TODO : Check if we really do both audio and video !
-
+        self.debug("Creating encoding thread")
         if not settings:
-            setting = self.getSettings()
+            settings = self.getSettings()
             if not settings:
                 self.error("No settings available to create the Encoding Thread")
                 return None
@@ -300,15 +299,21 @@ class SmartBin(gst.Pipeline):
         ##
 
         ainq = gst.element_factory_make("queue", "ainq")
+        ainq.props.max_size_time = 5 * gst.SECOND
+        ainq.props.max_size_buffers = 0
+        ainq.props.max_size_bytes = 0
         aident = gst.element_factory_make("identity", "aident")
         aident.props.single_segment = True
         aconv = gst.element_factory_make("audioconvert", "aconv")
         ares = gst.element_factory_make("audioresample", "ares")
         arate = gst.element_factory_make("audiorate", "arate")
-        aenc = gst.element_factory_make(settings.aencoder ,"aenc")
-        # set properties on the encoder
-        for prop, value in settings.acodecsettings.iteritems():
-            aenc.set_property(prop, value)
+        if settings.aencoder:
+            aenc = gst.element_factory_make(settings.aencoder ,"aenc")
+            # set properties on the encoder
+            for prop, value in settings.acodecsettings.iteritems():
+                aenc.set_property(prop, value)
+        else:
+            aenc = gst.element_factory_make("identity", "aenc")
         aoutq = gst.element_factory_make("queue", "aoutq")
 
         # add and link all required audio elements
@@ -329,23 +334,25 @@ class SmartBin(gst.Pipeline):
         aghost.set_active(True)
         thread.add_pad(aghost)
 
-##         aenc.get_pad("sink").add_data_probe(self._debugProbe, "aenc-sink")
-##         aenc.get_pad("src").add_data_probe(self._debugProbe, "aenc-src")
-
         ##
         ## Video part
         ##
 
         vinq = gst.element_factory_make("queue", "vinq")
+        vinq.props.max_size_buffers = 0
+        vinq.props.max_size_bytes = 0
         vident = gst.element_factory_make("identity", "vident")
         vident.props.single_segment = True
         csp = gst.element_factory_make("ffmpegcolorspace", "csp")
         vrate = gst.element_factory_make("videorate", "vrate")
         vscale = SmartVideoScale()
-        venc = gst.element_factory_make(settings.vencoder, "venc")
-        # set properties on the encoder
-        for prop, value in settings.vcodecsettings.iteritems():
-            venc.set_property(prop, value)
+        if settings.vencoder:
+            venc = gst.element_factory_make(settings.vencoder, "venc")
+            # set properties on the encoder
+            for prop, value in settings.vcodecsettings.iteritems():
+                venc.set_property(prop, value)
+        else:
+            venc = gst.element_factory_make("identity", "venc")
         voutq = gst.element_factory_make("queue", "voutq")
 
         # add and link all required video elements
@@ -368,9 +375,6 @@ class SmartBin(gst.Pipeline):
         vghost.set_active(True)
         thread.add_pad(vghost)
 
-##         vrate.get_pad("sink").add_data_probe(self._debugProbe, "before-vrate")
-##         vrate.get_pad("src").add_data_probe(self._debugProbe, "after-vrate")
-
         thread.filesink = fsink
 
         return thread
@@ -396,7 +400,8 @@ class SmartFileBin(SmartBin):
                           has_video = factory.is_video,
                           has_audio = factory.is_audio,
                           width = width, height = height,
-                          length = factory.length)
+                          length = factory.getDuration(),
+                          is_seekable = True)
 
     def _addSource(self):
         self.add(self.source)
@@ -454,7 +459,8 @@ class SmartTimelineBin(SmartBin):
                           has_video=True, has_audio=True,
                           width=settings.videowidth,
                           height=settings.videoheight,
-                          length=project.timeline.videocomp.duration)
+                          length=project.timeline.videocomp.duration,
+                          is_seekable=True)
 
     def _addSource(self):
         self.add(self.source)
@@ -513,3 +519,180 @@ class SmartDefaultBin(SmartBin):
         self.videotestsrc.get_pad("src").link(self.vtee.get_pad("sink"))
         self.silence.get_pad("src").link(self.atee.get_pad("sink"))
         self.debug("finished connecting sources")
+
+
+#FIXME : ... seriously... WHAT is this for ???
+class SinkBin:
+    """
+    A general purpose sink with audio and video. This can be used as sink for Capture
+    classes and for testing. Source for this sink is a Pipeline.
+    """
+    def __init__(self):
+
+        self.videosink = plumber.get_video_sink()
+
+        self.vsinkthread = gst.Bin('vsinkthread')
+        vqueue = gst.element_factory_make('queue')
+        timeoverlay = gst.element_factory_make('timeoverlay')
+        cspace = gst.element_factory_make('ffmpegcolorspace')
+        vscale = gst.element_factory_make('videoscale')
+        vscale.props.method = 1
+        self.vsinkthread.add(self.videosink, timeoverlay, vqueue, vscale, cspace)
+        vqueue.link(timeoverlay)
+        timeoverlay.link(self.videosink)
+        timeoverlay.set_property("halign", "right")
+        timeoverlay.set_property("valign", "bottom")
+        timeoverlay.set_property("deltax", 65)
+        timeoverlay.set_property("deltay", 20)
+
+
+        cspace.link(vscale)
+        vscale.link(vqueue)
+        self.vsinkthread.videosink = self.videosink
+        self.vsinkthread.add_pad(gst.GhostPad("sink", cspace.get_pad('sink')))
+
+        gst.debug("Creating audio sink")
+        self.audiosink = plumber.get_audio_sink()
+        self.asinkthread = gst.Bin('asinkthread')
+        aqueue = gst.element_factory_make('queue')
+        aconv = gst.element_factory_make('audioconvert')
+        self.asinkthread.add(self.audiosink, aqueue, aconv)
+        aconv.link(aqueue)
+        aqueue.link(self.audiosink)
+        self.asinkthread.audiosink = self.audiosink
+        self.asinkthread.add_pad(gst.GhostPad("sink", aconv.get_pad('sink')))
+
+    def connectSink(self, player, is_video, is_audio):
+        if is_video is True :
+            player.setVideoSinkThread(self.vsinkthread)
+        if is_audio is True :
+            player.setAudioSinkThread(self.asinkthread)
+    gst.debug("success connecting sources to SinkBin")
+
+
+class SmartCaptureBin(SmartBin):
+    """
+    SmartBin derivative for capturing streams.
+    """
+
+    def __init__(self, audiodevice=None, videodevice=None):
+        gst.log("Creating new smartcapturebin")
+        self.__audiodev = audiodevice
+        self.__videodev = videodevice
+        self.videosrc = None
+        self.audiosrc = None
+
+        SmartBin.__init__(self, "smartcapturebin",
+                          has_video=bool(videodevice),
+                          has_audio=bool(audiodevice),
+                          width=640, height=490)
+
+
+    def _addSource(self):
+        if self.__audiodev:
+            self.audiosrc = self.__audiodev.makeAudioBin()
+            self.q2 = gst.element_factory_make("queue", "webcam-firstaqueue")
+            self.q2.props.max_size_time = 30 * gst.SECOND
+            self.q2.props.max_size_buffers = 0
+            self.q2.props.max_size_bytes = 0
+            self.add(self.audiosrc, self.q2)
+        if self.__videodev:
+            self.videosrc = self.__videodev.makeVideoBin()
+            self.q1 = gst.element_factory_make("queue", "webcam-firstvqueue")
+            self.q1.props.max_size_time = 10 * gst.SECOND
+            self.add(self.videosrc, self.q1)
+
+    def _connectSource(self):
+        self.debug("connecting sources")
+        if self.videosrc:
+            gst.element_link_many(self.videosrc, self.q1, self.vtee)
+        if self.audiosrc:
+            gst.element_link_many(self.audiosrc, self.q2, self.atee)
+        self.debug("finished connecting sources")
+
+    def _asyncReset(self, uri, setting):
+        self.set_state(gst.STATE_READY)
+
+        # FIXME : This is maybe a temporary hack.
+        #
+        # EXPLANATION : The problem is that alsasrc (or any other audio source) will
+        # not reset the timestamps when going down to READY, but v4l2src (or any
+        # other element that resets itself in READY) will properly reset the
+        # timestamps.
+        # The resulting behaviour (without this fix) is that v4l2src will output
+        # buffers starting from 0 whereas alsasrc will output buffers starting from
+        # the last outputted buffer timestamp
+        #
+        # This is fixed in gst-plugins-base cvs 0.10.20.1
+        self.debug("Setting sources to NULL again to reset their timestamps !")
+        if self.videosrc:
+            self.videosrc.set_state(gst.STATE_NULL)
+            self.videosrc.set_state(gst.STATE_READY)
+        if self.audiosrc:
+            self.audiosrc.set_state(gst.STATE_NULL)
+            self.audiosrc.set_state(gst.STATE_READY)
+
+        SmartBin.record(self, uri, settings)
+
+    # It makes the recording video lag
+    def record(self, uri, settings=None):
+
+        if gst.version() < (0, 10, 20, 1):
+            # Made threaded to resolve video/audio lag issue.
+            self.debug("Calling record asynchronously")
+            CallbackThread(self._asyncReset, uri, settings).start()
+        else:
+            SmartBin.record(self, uri, settings)
+
+
+
+class SmartStreamBin(SmartBin):
+    """
+    SmartBin derivative for capturing streams.
+    """
+
+    def __init__(self, uri):
+        gst.log("Creating new smartcapturebin")
+        (self.is_video, self.is_audio) = (True, True)
+        self.uri = uri
+        SmartBin.__init__(self, "smartcapturebin", has_video=self.is_video,
+                          has_audio=self.is_audio,
+                          width=640, height=480)
+
+
+    def _addSource(self):
+        self.urisrc = gst.element_make_from_uri(gst.URI_SRC,
+                                                self.uri)
+        self.decodebin = gst.element_factory_make("decodebin", "decode-smartbin")
+        self.videoq = gst.element_factory_make("queue", "video-queue")
+        self.audioq = gst.element_factory_make("queue", "audio-queue")
+
+        self.add(self.urisrc, self.decodebin, self.videoq, self.audioq)
+
+    def _connectSource(self):
+        self.debug("connecting sources")
+        gst.element_link_many(self.urisrc, self.decodebin)
+        if self.is_video :
+            gst.element_link_many(self.videoq, self.vtee)
+        if self.is_audio:
+            gst.element_link_many(self.audioq, self.atee)
+
+
+        self.decodebin.connect("new-decoded-pad",
+                               self.on_new_decoded_pad)
+
+        self.debug("finished connecting sources")
+
+    # DecodeBin callback
+    # FIXME : What guarantees do you have that it will have both audio
+    # and video ????
+    def on_new_decoded_pad(self, element, pad, last):
+        caps = pad.get_caps()
+        name = caps[0].get_name()
+        if name.startswith("video"):
+            pad.link(self.videoq.get_pad('sink'))
+
+        elif name.startswith("audio"):
+            pad.link(self.audioq.get_pad('sink'))
+
+
