@@ -55,7 +55,7 @@ class GlobalSettings:
     # * The settings should be discover-able by UI for ex
     # * plugins should be able to add some settings
 
-    def __init__(self):
+    def __init__(self, **unused_kw):
         self._setDefaults()
         self._readSettings()
 
@@ -81,10 +81,11 @@ class GlobalSettings:
         # reads some settings from environment variable
         #self.advancedModeEnabled =
         #get_bool_env("PITIVI_ADVANCED_MODE")
-        self.fileSupportEnabled = get_bool_env("PITIVI_FILE_SUPPORT")
+        #self.fileSupportEnabled = get_bool_env("PITIVI_FILE_SUPPORT")
+        self.fileSupportEnabled = True
         pass
 
-    def get_local_plugin_path(autocreate=True):
+    def get_local_plugin_path(self, autocreate=True):
         """
         Compute the absolute path to local plugin repository
 
@@ -93,16 +94,16 @@ class GlobalSettings:
         """
 
         pitivi_path = os.path.expanduser("~/.pitivi")
-        if not os.path.exists(pitivi_path):
+        if not os.path.exists(pitivi_path) and autocreate:
             os.mkdir(pitivi_path)
 
         repository_path = os.path.expanduser("~/.pitivi/plugins")
-        if not os.path.exists(repository_path):
+        if not os.path.exists(repository_path) and autocreate:
             os.mkdir(repository_path)
 
         return repository_path
 
-    def get_plugin_settings_path(autocreate=True):
+    def get_plugin_settings_path(self, autocreate=True):
         """
         Compute the absolute path to local plugin settings' repository
 
@@ -111,11 +112,11 @@ class GlobalSettings:
         """
 
         pitivi_path = os.path.expanduser("~/.pitivi")
-        if not os.path.exists(pitivi_path):
+        if not os.path.exists(pitivi_path) and autocreate:
             os.mkdir(pitivi_path)
 
         repository_path = os.path.expanduser("~/.pitivi/plugins-settings")
-        if not os.path.exists(repository_path):
+        if not os.path.exists(repository_path) and autocreate:
             os.mkdir(repository_path)
 
         return repository_path
@@ -145,7 +146,7 @@ class ExportSettings(gobject.GObject, Serializable):
     # TODO : Add PAR/DAR for video
     # TODO : switch to using GstFraction internally where appliable
 
-    def __init__(self):
+    def __init__(self, **unused_kw):
         gobject.GObject.__init__(self)
         self.videowidth = 720
         self.videoheight = 576
@@ -178,7 +179,10 @@ class ExportSettings(gobject.GObject, Serializable):
         astr = "width=%d,height=%d,pixel-aspect-ratio=%d/%d,framerate=%d/%d" % (self.videowidth, self.videoheight,
                                                                                 self.videopar.num, self.videopar.denom,
                                                                                 self.videorate.num, self.videorate.denom)
-        return gst.caps_from_string("video/x-raw-yuv,%s;video/x-raw-rgb,%s" % (astr, astr))
+        vcaps = gst.caps_from_string("video/x-raw-yuv,%s;video/x-raw-rgb,%s" % (astr, astr))
+        if self.vencoder:
+            return get_compatible_sink_caps(self.vencoder, vcaps)
+        return vcaps
 
     def getVideoDescription(self):
         """ Returns a human-readable markup-ed string describing the video properties """
@@ -193,8 +197,13 @@ class ExportSettings(gobject.GObject, Serializable):
 
     def getAudioCaps(self):
         """ Returns the GstCaps corresponding to the audio settings """
-        astr = "rate=%d,depth=%d,channels=%d" % (self.audiorate, self.audiodepth, self.audiochannels)
-        return gst.caps_from_string("audio/x-raw-int,%s;audio/x-raw-float,%s" % (astr, astr))
+        astr = "rate=%d,channels=%d" % (self.audiorate, self.audiochannels)
+        astrcaps = gst.caps_from_string("audio/x-raw-int,%s;audio/x-raw-float,%s" % (astr, astr))
+        if self.aencoder:
+            return get_compatible_sink_caps(self.aencoder, astrcaps)
+        return astrcaps
+
+        # interset with current audioencoder sink pad caps
 
     def setVideoProperties(self, width=-1, height=-1, framerate=-1, par=-1):
         """ Set the video width, height and framerate """
@@ -272,25 +281,48 @@ class ExportSettings(gobject.GObject, Serializable):
 
     def fromDataFormat(self, obj):
         Serializable.fromDataFormat(self, obj)
-        self.videowidth = ret["video-width"]
-        self.videoheight = ret["video-height"]
-        self.videorate = gst.Fraction(*ret["video-rate"])
-        self.videopar = gst.Fraction(*ret["video-par"])
+        self.videowidth = obj["video-width"]
+        self.videoheight = obj["video-height"]
+        self.videorate = gst.Fraction(*obj["video-rate"])
+        self.videopar = gst.Fraction(*obj["video-par"])
 
-        self.audiochannels = ret["audio-channels"]
-        self.audiorate = ret["audio-rate"]
-        self.audiodepth = ret["audio-depth"]
+        self.audiochannels = obj["audio-channels"]
+        self.audiorate = obj["audio-rate"]
+        self.audiodepth = obj["audio-depth"]
 
         # FIXME : check if the given encoder/muxer are available
-        self.vencoder = ret["video-encoder"]
-        self.aencoder = ret["audio-encoder"]
-        self.muxer = ret["muxer"]
-        if "container-settings" in ret:
-            self.containersettings = ret["container-settings"]
-        if "audio-encoder-settings" in ret:
-            self.acodecsettings = ret["audio-encoder-settings"]
-        if "video-encoder-settings" in ret:
-            self.vcodecsettings = ret["video-encoder-settings"]
+        self.vencoder = obj["video-encoder"]
+        self.aencoder = obj["audio-encoder"]
+        self.muxer = obj["muxer"]
+        if "container-settings" in obj:
+            self.containersettings = obj["container-settings"]
+        if "audio-encoder-settings" in obj:
+            self.acodecsettings = obj["audio-encoder-settings"]
+        if "video-encoder-settings" in obj:
+            self.vcodecsettings = obj["video-encoder-settings"]
+
+def get_compatible_sink_caps(factoryname, caps):
+    """
+    Returns the compatible caps between 'caps' and the sink pad caps of 'factoryname'
+    """
+    gst.log("factoryname : %s , caps : %s" % (factoryname, caps.to_string()))
+    factory = gst.registry_get_default().lookup_feature(factoryname)
+    if factory == None:
+        gst.warning("%s is not a valid factoryname" % factoryname)
+        return None
+
+    res = []
+    sinkcaps = [x.get_caps() for x in factory.get_static_pad_templates() if x.direction == gst.PAD_SINK]
+    for c in sinkcaps:
+        gst.log("sinkcaps %s" % c.to_string())
+        inter = caps.intersect(c)
+        gst.log("intersection %s" % inter.to_string())
+        if inter:
+            res.append(inter)
+
+    if len(res) > 0:
+        return res[0]
+    return None
 
 def list_compat(a, b):
     for x in a:
@@ -350,3 +382,10 @@ def encoders_muxer_compatible(encoders, muxer):
                 res.append(encoder)
                 break
     return res
+
+def muxer_can_sink_raw_audio(muxer):
+    return my_can_sink_caps(muxer, gst.Caps("audio/x-raw-float;audio/x-raw-int"))
+
+def muxer_can_sink_raw_video(muxer):
+    return my_can_sink_caps(muxer, gst.Caps("video/x-raw-yuv;video/x-raw-rgb"))
+
