@@ -22,7 +22,6 @@
 
 import os.path
 import gobject
-import time
 import gtk
 import gst
 import gst.interfaces
@@ -35,17 +34,7 @@ import pitivi.dnd as dnd
 from pitivi.signalgroup import SignalGroup
 
 from gettext import gettext as _
-
-def time_to_string(value):
-    if value == -1:
-        return "--:--:--.---"
-    ms = value / gst.MSECOND
-    sec = ms / 1000
-    ms = ms % 1000
-    mins = sec / 60
-    sec = sec % 60
-    hours = mins / 60
-    return "%02d:%02d:%02d.%03d" % (hours, mins, sec, ms)
+from pitivi.utils import time_to_string
 
 class PitiviViewer(gtk.VBox):
     """ Pitivi's viewer widget with controls """
@@ -75,7 +64,7 @@ class PitiviViewer(gtk.VBox):
         # signal for timeline duration changes : (composition, sigid)
         self._timelineDurationChangedSigId = (None, None)
 
-        self._addTimelineToPlayground()
+
 
     def _connectToProject(self, project):
         """Connect signal handlers to a project.
@@ -88,6 +77,10 @@ class PitiviViewer(gtk.VBox):
                                      None, self._tmpIsReadyCb)
         self.project_signals.connect(project, "settings-changed",
                                      None, self._settingsChangedCb)
+        # we should add the timeline to the playground here, so
+        # that the new timeline bin will be added to the
+        # playground when the project loads
+        self._addTimelineToPlayground()
 
     def _createUi(self):
         """ Creates the Viewer GUI """
@@ -306,7 +299,8 @@ class PitiviViewer(gtk.VBox):
     ## active Timeline calllbacks
 
     def _asyncTimelineDurationChanged(self, duration):
-        gst.debug("duration : %d" % duration)
+        gst.debug("duration : %s" % gst.TIME_ARGS(duration))
+        gst.debug("playground.current.length : %s" % gst.TIME_ARGS(instance.PiTiVi.playground.current.length))
         self.posadjust.upper = float(duration)
         self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time), time_to_string(instance.PiTiVi.playground.current.length)))
 
@@ -318,7 +312,7 @@ class PitiviViewer(gtk.VBox):
 
     def _timelineDurationChangedCb(self, unused_composition, unused_start,
                                    duration):
-        gst.debug("duration : %d" % duration)
+        gst.debug("duration : %s" % gst.TIME_ARGS(duration))
         if duration == 0:
             gobject.idle_add(self._backToDefaultCb)
         gobject.idle_add(self._asyncTimelineDurationChanged, duration)
@@ -352,8 +346,14 @@ class PitiviViewer(gtk.VBox):
         self._connectToProject(project)
 
     def _addTimelineToPlayground(self):
-        instance.PiTiVi.playground.addPipeline(instance.PiTiVi.current.getBin())
-
+        # remove old timeline before proceeding
+        pg = instance.PiTiVi.playground
+        timeline = pg.getTimeline()
+        if timeline:
+            pg.switchToDefault()
+            pg.removePipeline(timeline)
+        # add current timeline
+        pg.addPipeline(instance.PiTiVi.current.getBin())
 
     ## Control gtk.Button callbacks
 
@@ -385,7 +385,9 @@ class PitiviViewer(gtk.VBox):
 
     def _currentPlaygroundChangedCb(self, playground, smartbin):
         gst.log("smartbin:%s" % smartbin)
-        if smartbin == playground.default:
+
+        if not smartbin.seekable:
+            # live sources or defaults, no duration/seeking available
             self.slider.set_sensitive(False)
             self.playpause_button.set_sensitive(False)
             self.next_button.set_sensitive(False)
@@ -404,7 +406,7 @@ class PitiviViewer(gtk.VBox):
                 self._timelineDurationChangedSigId = (smartbin.project.timeline.videocomp,
                                                       sigid)
             else:
-                self.posadjust.upper = float(smartbin.factory.length)
+                self.posadjust.upper = float(smartbin.factory.getDuration())
                 if not self._timelineDurationChangedSigId == (None, None):
                     obj, sigid = self._timelineDurationChangedSigId
                     obj.disconnect(sigid)
@@ -430,7 +432,8 @@ class PitiviViewer(gtk.VBox):
             gst.log('%s' % dav)
             if dav and dav.realsink and dav.realsink == message.src:
                 self.drawingarea.can_set_xid = True
-                self.drawingarea.set_xwindow_id()
+                if self.drawingarea.isexposed:
+                    self.drawingarea.set_xwindow_id()
 
 
 class ViewerWidget(gtk.DrawingArea):
@@ -447,11 +450,15 @@ class ViewerWidget(gtk.DrawingArea):
         self.can_set_xid = False
         self.unset_flags(gtk.DOUBLE_BUFFERED)
         self.unset_flags(gtk.SENSITIVE)
+        self.isexposed = False
 
     def do_expose_event(self, unused_event):
         """ 'expose-event' override """
+        gst.log("expose have_set_xid:%d can_set_xid:%d" % (self.have_set_xid, self.can_set_xid))
+        self.isexposed = True
         if self.videosink:
             if not self.have_set_xid and self.can_set_xid:
+                gtk.gdk.display_get_default().sync()
                 self.set_xwindow_id()
             elif self.have_set_xid:
                 self.videosink.expose()
@@ -468,7 +475,7 @@ class ViewerWidget(gtk.DrawingArea):
                                                  self.window.xid))
         self.videosink.set_xwindow_id(self.window.xid)
         self.have_set_xid = True
-        self.videosink.expose()
+        #self.videosink.expose()
 
 
 class PlayPauseButton(gtk.Button):
@@ -507,5 +514,3 @@ class PlayPauseButton(gtk.Button):
             self.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
             self.playing = True
 
-
-
